/*
 
           
            from Avram K. Tetewsky, atetewsky@draper.com
         617-258-1287 voice, 617-258-4444 fax.   home 617-784-2845
         draper laboratory or charles stark draper laboratory, 
         555 Technology Square MS7C, 
         Cambridge Mass 02139.
         "Document unto others as you would document onto yourself" a tetewsky 1987
         "How did G-D created the world in 6 days? -- no documenation" a tetewsky 1987
 *
 *
 * file: lvccin1.C  Testable and Debugable Code Interface Node 1
   date_created:  11-09-93
   date_modified: 11-09-93

 */
#define TEST_HARNESS    0  /* set to zero for use in LabView, 1 in testing */
#define SUBSCRIPT_CHECK 1  /* set to zero for speed in LabView, 1 for in-line test */ 


#if( TEST_HARNESS == 1)
/******************* for the test harness **************/
   #include <stdio.h>
   #include <stdlib.h>
   #include <math.h>
   #include "FAKELV0.H"
/*******************************************************/
#else
   #include "EXTCODE.H"
#endif

#include "LVCCIN1.H"
#include "VERBOSE.H"


#define             MAX_INRATE  100     /* max input sampling rate */
#define             MAX_TLOOK_S 100     /* max second averaging time  */
#define MAX_SIZE    MAX_INRATE*MAX_TLOOK_S

static float64  data_buffer[MAX_SIZE],data2_buffer[MAX_SIZE];
static float64  raw_mean,the_mean,raw_rms2,the_rms2,scale,the_std2,the_std;
static int32    start;
static int32    total_error;

#if(TEST_HARNESS==1)
/******************* for the test harness **************/
#define NTIMES   2000000
main(int argc,char **argv)
begin_code
  int32     jj,some_error=0,some_size,some_tlook_s=1,actual_size;
  int32     inrate=10;
  float64   input=0.0,some_mean=0.0,some_std=0.0,pi,the_time=0.0;
  pi = 4.0*atan(1.0);
  some_size = some_tlook_s*inrate;
  printf(" === LVCCIN1.C LabView ControlCalc CIN 1 \n");
  printf(" ... MAX_INRATE=%d MAX_TLOOK_S=%d \n",MAX_INRATE,MAX_TLOOK_S);
  printf(" ... MAX_SIZE=%d  \n",MAX_SIZE);
  printf(" ... inrate=%d some_tlook_s=%d some_size=%d\n",
      inrate,some_tlook_s,some_size);
  printf(" ... pi=%e \n",pi);
  printf(" ... get ready to call Init and then Run NTIMES=%d times \n",NTIMES);
  CINInit();
  for(jj=0;jj<NTIMES;jj++) begin_for
     input=sin(2.0*pi*1.0*the_time)+1.0;
     CINRun(&input,&some_size,&some_mean,&some_std,&actual_size,&some_error);
     the_time += 1.0/inrate;
  end_for
  printf(" some_size=%d actual_size=%d MAX_SIZE=%d\n",some_size,actual_size,
           MAX_SIZE);
  printf(" mean=%e std=%e some_error=%d \n",some_mean,some_std,some_error);
  return noErr;
end_code
/**********************************************************/
#endif


CIN MgErr CINInit(void)
begin_code
  int32   jj;
  
  total_error = 0;
  start = 0;
  raw_mean = 0.0; raw_rms2 = 0.0;
  #if(TEST_HARNESS == 1)
/******************* for the test harness ************/

      printf(" ... CINInit  start=%d MAX_SIZE=%d \n",
             start,MAX_SIZE);
/*****************************************************/  
  #endif
  for(jj=0;jj<MAX_SIZE;jj++) begin_for
     #if(SUBSCRIPT_CHECK == 1)
        if( jj < 0 OR jj >= MAX_SIZE) then_part
            total_error += 1;
        end_then
     #endif
     data_buffer[jj]=0; data2_buffer[jj]=0;
  end_for
  return noErr;
end_code 

CIN MgErr CINRun(float64 *din_in,
                 int32   *size_in,
                 float64 *mean_out,
                 float64 *var_out,
                 int32   *size_out,
                 int32   *ierr_out) 
begin_code
     int32  new_start,new_end,the_size;
     if( *size_in < 1 OR *size_in > MAX_SIZE) then_part
         the_size = MAX_SIZE;
     end_then
     else else_part
         the_size = *size_in;
     end_else
     *size_out = the_size;
     scale     = 1.0/((float64) (the_size));
     new_start = (start+1) % the_size;
     raw_mean  -=  data_buffer[start];
     raw_rms2  -=  data2_buffer[start];
     data_buffer[start]=*din_in;
     data2_buffer[start]=data_buffer[start]*data_buffer[start];
     raw_mean += data_buffer[start];
     raw_rms2 += data2_buffer[start];
     
     start    = new_start;
     the_mean = raw_mean*scale;
     the_rms2 = raw_rms2*scale;
     the_std2 = the_rms2 - the_mean*the_mean;
     if( the_std2 >= 0.0) then_part
        the_std = (float64) sqrt( (double ) (the_std2) );
     end_then
     else else_part
        the_std = 0.0;
        total_error += 1;
        #if( TEST_HARNESS == 1) 
        /************************* for the test harness *********/
           printf(" !!! LVCCIN1.C:CINRUN neg sqrt \n");
        /********************************************************/
        #endif
     end_else
     *mean_out = the_mean;
     *var_out  = the_std;
     *ierr_out = total_error;
     return noErr;
end_code


CIN MgErr CINDispose(void) {return noErr;}
CIN MgErr CINAbort(void) {return noErr;}
 
CIN MgErr CINLoad(RsrcFile rf) {return noErr;}
CIN MgErr CINUnload(void) {return noErr;}
CIN MgErr CINSave(RsrcFile rf) {return noErr;}
